#include <AppKit/AppKit.h>
#include "TextView.h"
#include "defines.h"

@implementation TextView
- (BOOL)isFlipped
{
	return _flipped;
}

- (BOOL)isOpaque
{
	return YES;
}

- (void)drawRect:(NSRect) rect
{
	NSRange glyphRange;

	if (_flipped)
		[[NSColor redColor] set];
	else
		[[NSColor blueColor] set];
	NSRectFill([self bounds]);

	[[NSColor blackColor] set];

	// resize handling logic
	[_textContainer setContainerSize:[self bounds].size];
	glyphRange = [_layoutManager glyphRangeForTextContainer:_textContainer];

	// redraw logic
	[_layoutManager drawBackgroundForGlyphRange:glyphRange
					atPoint:rect.origin];
	[_layoutManager drawGlyphsForGlyphRange:glyphRange
					atPoint:rect.origin];
}

- (id)initWithFrame:(NSRect)frameRect
	 andStorage:(NSTextStorage *)storage
	 isFlipped:(BOOL)flipped
{
	_flipped = flipped;	// Preceed GS caching

	if (![super initWithFrame:(NSRect)frameRect])
		return nil;

	[self setAutoresizingMask:NSViewWidthSizable];

	_textStorage = storage;

	_layoutManager = [[NSLayoutManager alloc] init];
	_textContainer = [[NSTextContainer alloc]
		initWithContainerSize:NSMakeSize(500,250)];

#ifndef USE_SCREEN_FONTS
	[_layoutManager setUsesScreenFonts:NO];
#endif

	[_layoutManager addTextContainer:_textContainer];
	[_textContainer release];

	[_textStorage addLayoutManager:_layoutManager];
	[_layoutManager release];

	return self;
}
@end
