#include <AppKit/AppKit.h>
#include <math.h>
#include "text.h"
#include "defines.h"

static void
ts_add_str_with_font (NSTextStorage *ts, NSFont *font, NSString *str)
{
	NSAttributedString *as;

	if (font)
		as = [[NSAttributedString alloc]
			initWithString:str
			    attributes:[NSDictionary dictionaryWithObjectsAndKeys:
					font, NSFontAttributeName,
					nil, nil]];
	else
		as = [[NSAttributedString alloc]
			initWithString:[@"Fail: " stringByAppendingString:str]];
	[ts appendAttributedString:as];
	[as release];
}

void load_text_storage (NSTextStorage *ts)
{
	NSString *nm;
	NSFont *var_r_font;
	NSFont *var_o_font;
	NSMutableParagraphStyle *paraStyle;
	NSRange range;
	unsigned length;
	float mset[6];
	float mnew[6];

	//var_r_font = [NSFont userFixedPitchFontOfSize:12];
	var_r_font = [NSFont userFontOfSize:12];
	ts_add_str_with_font(ts, var_r_font, @"Regular text\n");

	memcpy(mset, [var_r_font matrix], sizeof mset);
	nm = [var_r_font familyName];
	NSLog(@"Got family<%@> name<%@>; [%f %f %f  %f %f %f]",
		[var_r_font familyName], [var_r_font fontName],
		mset[0], mset[1], mset[2], mset[3], mset[4], mset[5]);

	/* scaling (width) */

	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = mset[0] * 2; mnew[3] = mset[3];
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Scaled text width.\n");

	/* scaling (height) */
	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = mset[0]; mnew[3] = mset[3] * 2;
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Scaled text height.\n");

	/* scaling (equal) */
	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = mset[0] * 2; mnew[3] = mset[3] * 2;
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Scaled text.\n");

	/* oblique (right) */
	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = mset[0]; mnew[3] = mset[3];
	mnew[2] = mnew[3] * tan(8.0 * (3.1415927 / 90.0));;
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Oblique text right; ");

	/* oblique (left) */
	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = mset[0]; mnew[3] = mset[3];
	mnew[2] = -mnew[3] * tan(8.0 * (3.1415927 / 90.0));;
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Oblique text left.\n");

	/* inverted letters */
	ts_add_str_with_font(ts, var_r_font, @"Pad\n");
	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = mset[0]; mnew[3] = -mset[3];
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"____Inverted letter text.\n");

#ifndef NO_SLOPE_ROTATE
	/* sloped (up) */
	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = 2 * mset[0]; mnew[3] = 2 * mset[3];
	mnew[1] = mnew[3] * tan(2.5 * (3.1415927 / 90.0));;
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Sloped text up; ");

	/* rotated letters */
	memcpy(mnew, mset, sizeof mnew);
	{
		double angle_rad = 2.5 * (3.1415927 / 90.0);
		float cos_theta = cos(angle_rad);
		float sin_theta = sin(angle_rad);
		mnew[0] = 2 * mset[0] * cos_theta;
		mnew[1] = 2 * mset[0] * +sin_theta;
		mnew[2] = 2 * mset[3] * -sin_theta;
		mnew[3] = 2 * mset[3] * cos_theta;
	}
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Rotated letters.\n");
#endif

	ts_add_str_with_font(ts, var_r_font, @"\n");

	/* superscript */
	ts_add_str_with_font(ts, var_r_font, @"  Baseline; ");
	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = 1 * mset[0]; mnew[3] = 1 * mset[3];
	mnew[5] = mnew[3] * 1.5;
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Raised\n");

	/* left aligned paras */
	length = [ts length];
	range = NSMakeRange(0, length);

	paraStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
	[paraStyle setAlignment:NSLeftTextAlignment];
	[paraStyle setLineBreakMode:NSLineBreakByCharWrapping];
	[ts addAttribute:NSParagraphStyleAttributeName value:paraStyle range:range];

	/* backward letters */
	memcpy(mnew, mset, sizeof mnew);
	mnew[0] = -mset[0]; mnew[3] = mset[3];
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Backward letter text.\n");

	/* vertical */
	mnew[0] = 0;
	mnew[1] = 2 * mset[3];
	mnew[2] = -2 * mset[0];
	mnew[3] = 0;
	// tX and tY
	// Cairo stacks the letters vertically (i.e. the text baseline has
	// been rotated;  it uses tX to offset the horizontal position of
	// the stack.  It uses -tY to offset the vertical position of the
	// stack.  However since the angle or rotation is wrong,  then it
	// is probably actually adding tY along the (incorrect) direction.
	// OPENSTEP preserves the horizontal baseline.  It uses tX as character
	// seperation horizontally.
	mnew[4] = 18; // tX
	mnew[5] = -9; // tY
	var_o_font = [NSFont fontWithName:nm matrix:mnew];
	ts_add_str_with_font(ts, var_o_font, @"Vert\n");

	/* right aligned paras */
	range = NSMakeRange(length, [ts length] - length);

	/* cairo shows vertical text if this is also left aligned */
	paraStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
	//[paraStyle setAlignment:NSCenterTextAlignment];
	[paraStyle setAlignment:NSCenterTextAlignment];
	//[paraStyle setAlignment:NSLeftTextAlignment];
	//[paraStyle setAlignment:NSRightTextAlignment];
	[paraStyle setLineBreakMode:NSLineBreakByCharWrapping];
	[ts addAttribute:NSParagraphStyleAttributeName value:paraStyle range:range];

	/* common attrs */
	length = [ts length];
	range = NSMakeRange(0, length);

	[ts addAttribute:NSForegroundColorAttributeName value:[NSColor whiteColor] range:range];
	[ts addAttribute:NSLigatureAttributeName value:[NSNumber numberWithInt:0] range:range];
}
